#!/usr/bin/python2
# -*- coding: utf-8 -*-

import csv, sys, codecs
from pysqlite2 import dbapi2 as sqlite3
import cStringIO

class UTF8Recoder:
	"""
	Iterator that reads an encoded stream and reencodes the input to UTF-8
	"""
	def __init__(self, f, encoding):
		self.reader = codecs.getreader(encoding)(f)
		
	def __iter__(self):
		return self
		
	def next(self):
		return self.reader.next().encode("utf-8")
		
class UnicodeReader:
	"""
	A CSV reader which will iterate over lines in the CSV file "f",
	which is encoded in the given encoding.
	"""

	def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
		f = UTF8Recoder(f, encoding)
		self.reader = csv.reader(f, dialect=dialect, **kwds)

	def next(self):
		row = self.reader.next()
		return [unicode(s, "utf-8") for s in row]

	def __iter__(self):
		return self

def findMatches(kanji, kana):
	# Look for kanji/kana combinations
	r = set()
	for kj in kanji:
		for kn in kana:
			qString = "select kana.id from kanaText join kana on kana.docid = kanaText.docid join kanji on kana.id = kanji.id join kanjiText on kanji.docid = kanjiText.docid join entries on entries.id = kana.id where kanaText.reading match ? and kanjiText.reading match ? order by kanji.priority asc, entries.frequency desc"
			r = r.union(set([e[0] for e in cursor.execute(qString, (kn, kj)).fetchall()]))
	# Look for kana only combinations
	if len(r) == 0 and len(kanji) == 0:
		# Find word that can only be written in kanji
		for kn in kana:
			qString = "select kana.id from kanaText join kana on kana.docid = kanaText.docid left join kanji on kana.id = kanji.id join entries on entries.id = kana.id join senses on senses.id == kana.id where kanaText.reading match ? and (kanji.docid is null or (senses.misc0 & 4 != 0)) order by entries.frequency desc"
			r = r.union(set([e[0] for e in cursor.execute(qString, (kn,)).fetchall()]))
	# Find anything matching the kana in despair
	if (len(r) == 0):
		for kn in kana:
			qString = "select kana.id from kanaText join kana on kana.docid = kanaText.docid join entries on entries.id = kana.id where kanaText.reading match ? order by entries.frequency desc"
			r = r.union(set([e[0] for e in cursor.execute(qString, (kn,)).fetchall()]))
	return r

if __name__ == "__main__":
	csvreader = UnicodeReader(codecs.open(sys.argv[1], 'r'), delimiter=',', quotechar='"')
	out = open(sys.argv[1][:-3] + "out", "w")
	connection = sqlite3.connect(sys.argv[2])
	connection.enable_load_extension(True)
	cursor = connection.cursor()
	# Load tagaini extensions
	cursor.execute("select load_extension('../../../build/src/sqlite/libtagaini_sqlite.so')")
	for row in csvreader:
		# Splits
		kanji = row[0]
		kana = row[1]
		kanji = kanji.replace(u"・", "/")
		kana = kana.replace(u"・", "/")
		kanji = kanji.replace(u"、", "/")
		kana = kana.replace(u"、", "/")
		kanji = kanji.replace(u" ", "/")
		kana = kana.replace(u" ", "/")
		kanji = [r.strip() for r in kanji.split("/")]
		kana = [r.strip() for r in kana.split("/")]
		if len(kanji) == 1 and len(kanji[0]) == 0: kanji = []

		r = findMatches(kanji, kana)

		# Remove ending "suru"
		for i in range(len(kanji)):
			k = kanji[i]
			if k.endswith(u"する"): kanji[i] = k[:-2]
		for i in range(len(kana)):
			k = kana[i]
			if k.endswith(u"する"): kana[i] = k[:-2]
		r = r.union(findMatches(kanji, kana))

		# Look by removing final "to"
		for i in range(len(kana)):
			k = kana[i]
			if k.endswith(u"と"): kana[i] = k[:-1]
		for i in range(len(kanji)):
			k = kanji[i]
			if k.endswith(u"と"): kanji[i] = k[:-1]
		r = r.union(findMatches(kanji, kana))

		for i in r:
			out.write("%d\n" % (i,))

		if len(r) == 0:
			if len(kanji) > 0: print "Nothing found for", kana[0], kanji[0]
			else: print "Nothing found for", kana[0]



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#  Copyright (C) 2008  Alexandre Courbot
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import codecs
import sys
import re
import os
from PyQt4 import QtCore
from PyQt4.QtSql import *
from database import getDb

# Database connection name used to access our custom driver
dbConnectionName = "custom-sqlite"

def addJLPTLevels(dbFile, jlptFile, level, outFileName):
	global db, dbConnectionName

	outFile = codecs.open(outFileName, 'w', 'utf-8')

	db = getDb(dbConnectionName)
	db.setDatabaseName(dbFile)
	if not db.open():
		usage()
		sys.exit(1)
	query = QSqlQuery(db)

	re1 = QtCore.QRegExp(u'~?([\w]+)~?\s+\\[?~?([\w]+)~?\\]?.*')
	re2 = QtCore.QRegExp(u'~?([\w]+)~?.*')
	file = open(jlptFile, "r")
	for line in file.xreadlines():
		if len(line.strip()) == 0 or line[0] == '#': continue
		s = QtCore.QString.fromUtf8(line)
		if re1.exactMatch(s):
			matches = re1.capturedTexts()
			kanji = unicode(matches[1])
			kana = unicode(matches[2])
			qString = "select kana.id from kanaText join kana on kana.docid = kanaText.docid join kanji on kana.id = kanji.id join kanjiText on kanji.docid = kanjiText.docid join entries on entries.id = kana.id where kanaText.reading match ? and kanjiText.reading match ? order by kanji.priority asc, entries.frequency desc"
			query.prepare(qString)
			query.addBindValue(QtCore.QVariant(kana))
			query.addBindValue(QtCore.QVariant(kanji))
		elif re2.exactMatch(s):
			matches = re2.capturedTexts()
			kanji = None
			kana = unicode(matches[1])
			qString = "select kana.id from kanaText join kana on kana.docid = kanaText.docid join entries on entries.id = kana.id where kanaText.reading match ? order by entries.frequency desc"
			query.prepare(qString)
			query.addBindValue(QtCore.QVariant(kana))
		else:
			print "Unmatched line: " + line,
			continue

		if not query.exec_(): print query.lastError().text(); raise Exception
		if not query.next(): print "No result for %s %s" % (kanji, kana); continue
		outFile.write("%d\n" % (query.value(0).toInt()[0]))
		if query.next(): print "Multiple results for %s %s" % (kanji, kana)

if __name__ == "__main__":
	if len(sys.argv) != 5: usage()
	else: 
		addJLPTLevels(sys.argv[1], sys.argv[2], int(sys.argv[3]), sys.argv[4])



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#  Copyright (C) 2008  Alexandre Courbot
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import codecs
import sys
import re
import os
from PyQt4 import QtCore
from PyQt4.QtSql import *
from database import getDb

# Database connection name used to access our custom driver
dbConnectionName = "custom-sqlite"

# Tries to automatically associate a JLPT level to JMdict entries, according to the information available on
# http://www.jlptstudy.com. Ambigous or unknown entries are left for manual inclusion.
def addJLPTLevels(dbFile, jlptFile, level, outFile):
	global db, dbConnectionName

	outFile = codecs.open(outFile, "w", "utf-8")

	db = getDb(dbConnectionName)
	db.setDatabaseName(dbFile)
	if not db.open():
		usage()
		sys.exit(1)
	query = QSqlQuery(db)
	
	nbpos = 0
	nben = 0

	levelRegExp = re.compile('''.*<td class="(.+)"(?:>| />)([^<$\n]*).*''')
	file = open(jlptFile, "r")
	for line in file.xreadlines():
		match = levelRegExp.match(line)
		if match:
			if match.group(1) == "pos":
				kana = None
				kanji = None
				en = None
				nbpos += 1
			elif match.group(1) == "kanji":
				if kana == None:
					kana = QtCore.QString.fromUtf8(match.group(2))
					kana.remove(u"～")
					kana.remove(u"。")
					kana.remove(u"・")
					kana.remove(u"、")
					idx = kana.indexOf("/")
					if idx >= 0: kana.truncate(idx)
#					idx = kana.indexOf(u"・")
#					if idx >= 0: kana.truncate(idx)
					kana.remove(QtCore.QRegExp(u"（.*）"))
					kana.remove(QtCore.QRegExp(u"\(.*\)"))
					kana.remove(QtCore.QRegExp(u"する$"))
				else:
					kanji = QtCore.QString.fromUtf8(match.group(2))
					kanji.remove(u"～")
					idx = kanji.indexOf("/")
					if idx >= 0: kanji.truncate(idx)
			elif match.group(1) == "en":
				nben += 1
				en = QtCore.QString.fromUtf8(match.group(2))
				if kana != None:
					qString = "select kana.id from kanaText join kana on kana.docid = kanaText.docid join kanji on kana.id = kanji.id join kanjiText on kanji.docid = kanjiText.docid join entries on entries.id = kana.id where kanaText.reading match ? and kanjiText.reading match ? order by entries.frequency desc"
					query.prepare(qString)
					query.addBindValue(QtCore.QVariant(kana))
					query.addBindValue(QtCore.QVariant(kanji))
					if not query.exec_(): print query.lastError().text(); raise Exception
					# No result? Let's try by looking at the kana and definition then.
					if not query.next():
						query.prepare("select kana.id from kanaText join kana on kana.docid = kanaText.docid join gloss on gloss.id = kana.id join glossText on gloss.docid = glossText.docid join entries on entries.id = kana.id where kanaText.reading match ? and glossText.reading match ? order by entries.frequency desc")
						query.addBindValue(QtCore.QVariant(kana))
						query.addBindValue(QtCore.QVariant(en))
						if not query.exec_(): print query.lastError().text(); raise Exception
						if not query.next():
							# Still not? Then try kana and accept if there is only one entry corresponding
							query.prepare("select kana.id from kanaText join kana on kana.docid = kanaText.docid join entries on entries.id = kana.id where kanaText.reading match ? order by entries.frequency desc")
							query.addBindValue(QtCore.QVariant(kana))
							if not query.exec_(): print query.lastError().text(); raise Exception
							if not query.next():
								outFile.write("############%s %s\n" % (unicode(kanji), unicode(kana)))
								continue
							if query.next():
								outFile.write("############%s %s\n" % (unicode(kanji), unicode(kana)))
								continue
							query.first() 

					# Make sure not to violate primary key constraint
					#query2.prepare("insert or ignore into jlpt values (?, %d)" % (4))
					#query2.addBindValue(QtCore.QVariant(query.value(0)))
					#if not query2.exec_(): print query2.lastError().text(); raise Exception
					outFile.write("%d\n" % (query.value(0).toInt()[0]))
			else: print "Bad entry: %s" % (match.group(1))
	print nbpos, nben

if __name__ == "__main__":
	if len(sys.argv) != 5: usage()
	else: 
		addJLPTLevels(sys.argv[1], sys.argv[2], int(sys.argv[3]), sys.argv[4])
